<?php
	/*
	Plugin Name: Instapress
	Plugin URI: http://wordpress.org/extend/plugins/instapress/
	Description: Widget to display Instagram-Pics from a specific user.
	Version: 0.4.3
	Author: tkrammer
	Author URI: http://liechtenecker.at
	License: GPL2
	*/

	define('INSTAPRESS_VERSION', '0.4.3');

	/*  Copyright 2011 Thomas Krammer  (email : t.krammer@liechtenecker.at)

	    This program is free software; you can redistribute it and/or modify
	    it under the terms of the GNU General Public License, version 2, as 
	    published by the Free Software Foundation.
	
	    This program is distributed in the hope that it will be useful,
	    but WITHOUT ANY WARRANTY; without even the implied warranty of
	    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	    GNU General Public License for more details.
	
	    You should have received a copy of the GNU General Public License
	    along with this program; if not, write to the Free Software
	    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
	*/

	/**
	 * Sprachdatei laden
	 */
	if(!load_plugin_textdomain('instagram','/wp-content/languages/')) // Nach Sprachdatei im wp-languages-Ordner suchen 
		load_plugin_textdomain('instagram','/wp-content/plugins/instapress/languages/'); // Default Sprachdatei laden
		
		
	
	// Include-Path f�r Zend-Library setzen
	set_include_path(get_include_path().PATH_SEPARATOR.plugin_dir_path(__FILE__).'instagram-php-api/');
	
	require_once 'Instagram.php';

	class InstagramPlugin
	{
				
		// Key in der Tabelle wp_options, unter dem die Einstellungen gespeichert werden
		var $dbOptionKey = 'InstagramPlugin_Options';
		
		
		/**
		 * Constructor
		 */
		function InstagramPlugin()
		{
			// Men� im Backend hinzuf�gen
			add_action('admin_menu', array(&$this, 'admin_menu'));
			
			// Shortcode registrieren
			add_shortcode('instapress', array(&$this, 'shortcode'));
			
			// Link in der Plugins-Liste zu den Einstellungen
			add_filter('plugin_action_links', array(&$this, 'plugin_page_link'), 5, 2);
		}
		
		/**
		 * @return InstagramPlugin Die aktive Instanz des Plugins
		 */
		function getInstance()
		{
			global $InstagramPlugin;
			if(!isset($InstagramPlugin))
			{
				$InstagramPlugin = new InstagramPlugin();
			}
			
			return $InstagramPlugin;
		}
		
		/**
		 * @return Instagram eine konfigurierte Instanz der API
		 */
		function getAPIInstance()
		{
			// Konfiguration laden
			$config = InstagramPlugin::getConfiguration();
			        
			// API instanzieren
			$instagram = new Instagram($config);
			
			$instagram->setAccessToken(InstagramPlugin::getAccessToken());
			
			return $instagram;
		}
		
		/**
		 * Wird im register_actication_hook aufgerufen
		 */
		function install()
		{
			$this->getOptions();
		}
		
		/**
		 * Wird bei der Implementierung von shortcode aufgerufen
		 */
		function shortcode($params)
		{
			$values = shortcode_atts(array
									(
										'userid' => '',
										'size' => 85,
										'piccount' => 9,
										'effect' => false
									), 
									$params);
																		
			$data = InstagramPlugin::getFeedByUserId($values['userid']);
			
			// Default-Groesse ist 150x150
			$picSize = (intval($values['size']) > 0) ? intval($values['size']) : 150;
			
			$result = "";
			
			if(count($data) > 0)
			{
				foreach($data as $index=>$obj)
				{
					// Nur die erste X Bilder anzeigen?
					if(intval($values['piccount']) > 0 && $index+1 > $values['piccount'])
						break;
						
					// Zurzeit noch kein Support für den Image-Title im Shortcode
					$title = "";
						
					$result .= '<div class="instapress-shortcode">';
					
					// Auf aktivierte Effekte überprüfen
					switch($values['effect'])
					{
						// jQuery Fancybox
						case 'fancybox':
							$result .= '<a href="'.$obj->images->standard_resolution->url.'" rel="instagram-images" title="'.$title.'">';
							break;
						// Ohne Effekt
						default:
							$result .= '<a href="'.$obj->link.'" target="_blank">';
							break;
					}
					
					$result .= '<img src="'.$obj->images->thumbnail->url.'" width="'.$picSize.'" height="'.$picSize.'" border="0" /></a></div>';
				}
			}
			
			return $result;
		}
		
		/**
		 * L�dt den Feed f�r den angegebenen User
		 * @param $userid mixed User-Id, 'self' oder 0/null/false
		 * 
		 * @return array der Feed (siehe Instagram API Dokumentation)
		 */
		function getFeedByUserId($userid)
		{	
			// Wenn es eine User-Id gibt bzw. 'self' hinterlegt wurde, diesen Feed laden
			if(intval($userid) > 0 || $userid == 'self')
			{
				$json = InstagramPlugin::getAPIInstance()->getUserRecent($userid);
			}
			// Wenn statt der UserId 'myfeed' eingetragen wurde, den Feed des Users laden
			else if($userid == 'myfeed')
			{
				$json = InstagramPlugin::getAPIInstance()->getUserFeed();
			}
			// ansonsten einfach den Popular-Media-Feed laden
			else
			{
				$json = InstagramPlugin::getAPIInstance()->getPopularMedia();
			}
						
			$response = json_decode($json);
			
			return $response->data;
		}
		
		/**
		 * L�dt die gespeicherten Einstellungen bzw. die Standardeinstellungen
		 * 
		 * @return array Gespeicherte Einstellungen
		 */
		function getOptions()
		{
			// Default-Werte
			$options = array
			(
				'app_client_id' => '',
				'app_client_secret' => '',
				'app_access_token' => ''
			);
			
			// Gespeicherte Werte laden
			$saved = get_option($this->dbOptionKey);
			
			// Wenn es gespeicherte Werte gibt
			if(!empty($saved))
			{
				// Gespeicherte Werte �ber Default-Werte schreiben
				foreach($saved as  $key => $option)
				{
					$options[$key] = $option;
				}
			}
			
			//
			if($saved != $options)
				update_option($this->dbOptionKey, $options);
				
			return $options;
		}
		
		function getPluginUrl()
		{
			return get_admin_url(null, 'options-general.php?page=instagram.php');
		}
		
		/**
		 * Gets a single option value
		 * 
		 * @param $key string Key der Option
		 * 
		 * @return mixed
		 */
		function getOption($key)
		{
			$options = $this->getOptions();
			
			return $options[$key];
		}
		
		/**
		 * Handelt das Formular
		 */
		function handleOptions()
		{
			$options = $this->getOptions();
			
			//Formular abgesendet
			if(isset($_POST['instagram-update-settings']))
			{			
				$options = array();
				$options['app_client_id'] = trim($_POST['instagram-app-client-id']);
				$options['app_client_secret'] = trim($_POST['instagram-app-client-secret']);
				$options['app_access_token'] = '1481339.1a66081.e262587b49de4130ad4c13099806820c';
				
				// Einstellungen in der DB speichern
				update_option($this->dbOptionKey, $options);
				
				// Wenn es noch keinen Access-Token gibt
				if(!$options['app_access_token'])
				{
					echo '<div class="updated"><p>'.__('Please wait a moment, while your application is being authorized.', 'poetrycafe').'</p></div>';
					echo '<script type="text/javascript">window.location.href="https://api.instagram.com/oauth/authorize/?client_id='.$options['app_client_id'].'&redirect_uri='.urlencode($this->getOAuthRedirectUrl()).'&response_type=code";</script>';
				}
				else
				{
					echo '<div class="updated"><p>'.__('Settings saved.', 'poetrycafe').'</p></div>';
				}
			}
			// Einstellung zur�cksetzen
			else if(isset($_POST['instagram-reset-settings']))
			{
				// Einstellungen in der DB zur�cksetzten / l�schen
				delete_option($this->dbOptionKey);
			}
			// Access-Token holen
			else if(isset($_GET['code']))
			{			
				// Konfiguration laden
				$config = InstagramPlugin::getConfiguration();
				        
				// API instanzieren
				$instagram = new Instagram($config);
				// Wenn es noch keinen Token gibt
				if(!$options['app_access_token'])
				{
					// Request an das API schicken, um einen Access-Token zu erhalten
					$media = $instagram->getPopularMedia();
				}
				
				// Access-Token holen
				$token = $instagram->getAccessToken();
				
				// Wenn es einen Access-Token gibt
				if($token)
				{
					// Access-Token speichern
					$options['app_access_token'] = $token;
					// Einstellungen in der DB speichern
					update_option($this->dbOptionKey, $options);
				}
			}
			
			// Die Instagram-Authorize-URI
			$authorizeUrl = $this->getOAuthRedirectUrl();
			
			include('instagram-options.php');
		}
		
		/**
		 * @return array Gibt die Einstellungen f�r die API zur�ck
		 */
		function getConfiguration()
		{
			$options = InstagramPlugin::getInstance()->getOptions();
			return array(
							'site_url' 		=> 'https://api.instagram.com/oauth/access_token',
				            'client_id' 	=> $options['app_client_id'],
				            'client_secret' => $options['app_client_secret'],
				            'grant_type' 	=> 'authorization_code',
				            'redirect_uri'	=> InstagramPlugin::getOAuthRedirectUrl()
				        );
		}
		
		/**
		 * F�gt den Men�punkt Instapress im Backend hinzu
		 */
		function admin_menu()
		{
			add_options_page('Instapress '.__('Settings', 'instagram'), 'Instapress', 8, basename(__FILE__), array(&$this, 'handleOptions'));
		}
		
		/**
		 * @return string Name des Plugins
		 */
		function getPluginName()
		{
			return plugin_basename(__FILE__);
		}
		
		/**
		 * Hook f�r die Link in der Plugin-Liste
		 * @param $links
		 * @param $file
		 */
		function plugin_page_link($links, $file) 
		{			
			// Wenn es dieses Plugin ist
			if($file == $this->getPluginName())
			{
				// Link zu Einstellungs-Seite
				$settingsLink = '<a href="'.$this->getOAuthRedirectUrl().'">'.__('Settings', 'instagram').'</a>';
				// Vorne anh�ngen
				array_unshift( $links, $settingsLink );
			}
			
			return $links;
		}
		
		/**
		 * @return string die URI an die nach der Autorisierung weitergeleitet wird
		 */
		function getOAuthRedirectUrl()
		{
			return get_admin_url().'options-general.php?page=instagram.php';//.'instagram/oauth.php';
		}
		
		/**
		 * @return string Access-Token der in der Datenbank gespeichert ist bzw. null
		 */
		function getAccessToken()
		{
			$options = InstagramPlugin::getInstance()->getOptions();
			
			return $options['app_access_token'];
		}
		
		function getVersion()
		{
			return INSTAPRESS_VERSION;
		}
		
	}
		
	/**
	 * Widget registrieren und laden
	 */
	if(!function_exists('load_instagram')):		
		add_action( 'widgets_init', 'load_instagram' );
		function load_instagram() 
		{
			register_widget( 'Instagram_Widget' );
		}
	endif;
		
	/**
	 * Plugin instanzieren
	 */
	if (class_exists('InstagramPlugin')): 
		$InstagramPlugin = InstagramPlugin::getInstance();
		if (isset($InstagramPlugin)) 
		{
			register_activation_hook(__FILE__, array(&$InstagramPlugin, 'install'));
		}
	endif;
	

	include('widget.php');